
/****************************************************************************
 Module : amsultra_a2d_simple - An AMS-Ultra connection module
 
 Input :
        a : an analog input

 Output :
        d : digital output
 
This is a simple unidirectional connection module which is used for automatic connection
module insertion during elaboration. This module reads the analog value
coverts it to corresponding digital value.

Algorithm:
--------
The algorithm for exact value conversion is implemented by Ultrasim
using the VPI task

VPI task : $ultra_connect_a2d(a,d,tx,vth,vtl)
where

   d   = digital port of connect module
   a   = analog port of connect module
   vth = threshold voltage hi, above which a2d drives a logic 1'b1
   vtl = threshold voltage lo, below which a2d drives a logic 1'b0
    tx = time in which analog voltage lingers between the two 
         thresholds before a logic 1'bx is driven out to digital.

    The equivalent analog circuit presented to the ultrasim solver 
    by the $ultra_connect_a2d() function is an open circuit that 
    behaves as an ideal analog to digital converter, and no digital 
    load effect is modeled (neither resistive nor capacitive)

Handling of Driver-Receiver Segregation:
--------
This module performs a direct mapping from digital ordinary module 
drivers to ordinary module receivers, with no delay introduced.

Compatible Disciplines:
--------
This module is intended to be used for converting analog signals
with "electrical" discipline to digital signals with "logic" discipline.

Sensitivity and Timing:
--------
This is provided by various parameters passed to Ultrasim VPI task. 


Port Directions:
--------
   The input "a" is assumed to be an (input) analog port of discipline 
   "electrical", and the output "d" is assumed to be an (output) digital port 
   of discipline "logic".
 
****************************************************************************/
`include "disciplines.vams"
`timescale 1ns / 100ps

connectmodule amsultra_a2d_simple(a, d);
input a; output d;
electrical a; logic d;		
parameter real vsup=1.8 from (0:inf);          // nominal supply voltage
parameter real vthi=vsup/1.5 from (-inf:vsup); // upper input threshold
parameter real vtlo=vthi/2 from (-inf:vthi);   // lower threshold
parameter real vtol=vsup/100 from (0:(vthi-vtlo)/4]; // voltage tolerance
parameter real tr=0.2n from (0:1m);            // risetime (for defaults)
parameter real txdel=4*tr;                     // time midrange til output X
parameter real ttol=tr/4;                      // time tolerance of crossing

	assign d = d;
	initial
	    $ultra_connect_a2d(a, d, txdel, vthi, vtlo); 

endmodule

